# framework.py

from pygame.locals import *
import pygame, sys, os

states = []
dataPrefix = os.path.join("..", "data")
imagePrefix = os.path.join(dataPrefix, "images")
audioPrefix = os.path.join(dataPrefix, "audio")
magenta = (255, 0, 255)
maxFps = 30
black = (0, 0, 0)
white = (255, 255, 255)

def init(caption, scrW, scrH, fullscreen = False):
	global screen, keys, lFrame
	flags = 0
	pygame.init()
	if fullscreen:
		flags = pygame.FULLSCREEN
	screen = pygame.display.set_mode((scrW, scrH), flags)
	pygame.display.set_caption(caption)
	lFrame = pygame.time.get_ticks()
	keys = pygame.key.get_pressed()
	pygame.mouse.set_visible(False)

def addState(state):
	states.append(state)

def updateKeys():
	global keys, oldKeys
	
	oldKeys = keys
	pygame.event.pump()
	keys = pygame.key.get_pressed()
	
	for event in pygame.event.get():
		if event.type == QUIT:
			sys.exit()

def error(message):
	print "Error: " + message + "\n"
	sys.stdout.flush()
	sys.exit(1)

def mainLoop():
	for state in states:
		stopMusic()
		state.init()
		
		while not state.done:
			updateKeys()			
			state.input()
			
			state.update()
			
			state.draw()
			pygame.display.flip()
			corrFrameTime()

def fileExists(filename):
	return os.path.exists(filename)

def doScreenshot():
	for i in range(100):
		filename = "screenshot" + str(i+1) + ".bmp"
		if not fileExists(filename):
			pygame.image.save(screen, filename)
			break

def loadImage(filename, useAlpha = True):
	image = pygame.image.load(os.path.join(imagePrefix, filename)).convert()
	image.set_colorkey(magenta)
	return image

def loadSound(filename):
	filename = os.path.join(audioPrefix, filename)
	return pygame.mixer.Sound(filename)

def keyPressed(keyId, wasReleased = True):
	return keys[keyId] and not wasReleased or not oldKeys[keyId] and keys[keyId]

# Some gfx funcs to save some typing
def drawCentered(image, y=-1):
	global screen, scrDimensions
	
	if y == -1:
		y = (scrDimensions[1] - image.get_height()) / 2
	x = (scrDimensions[0] - image.get_width()) / 2
	
	rect = Rect(x, y, image.get_width(), image.get_height())
	
	screen.blit(image, rect)

def draw(image, x=0, y=0):
	global screen
	rect = Rect(x, y, image.get_width(), image.get_height())
	screen.blit(image, rect)

def drawTextOnSurf(font, text, color):
	return font.render(text, True, color)

def drawText(font, text, color, x, y):
	global screen
	surf = drawTextOnSurf(font, text, color)
	rect = Rect(x, y, surf.get_width(), surf.get_height())
	screen.blit(surf, rect)

def drawTextCentered(font, text, color, y=-1):
	global screen
	surf = drawTextOnSurf(font, text, color)
	x = (screen.get_width() - surf.get_width()) / 2
	if y == -1:
		y = (screen.get_height() - surf.get_height()) / 2
	rect = Rect(x, y, surf.get_width(), surf.get_height())
	screen.blit(surf, rect)
	return rect

def collide(rect1, rect2):
	return rect1.colliderect(rect2)

def pointColl(rect, point):
	return rect.collidepoint(point[0], point[1])

# Make sure we don't get above maxFps in framerate
def corrFrameTime():
	global maxFps, lFrame
	
	while pygame.time.get_ticks() - lFrame < 1000.0 / maxFps:
		pygame.time.delay(int(1000.0 / maxFps - (pygame.time.get_ticks() - lFrame)))

	lFrame = pygame.time.get_ticks()

def loadAndPlayMusic(filename, loops = -1):
	pygame.mixer.music.load(os.path.join(audioPrefix, filename))
	pygame.mixer.music.play(loops)

def stopMusic():
	pygame.mixer.music.stop()
